import { dataSource } from "../../data-source";
import { damageReport } from "../../entity/unit/damageReport";
import DatabaseActionException from "../../exceptions/databaseActionException";
import {
  CreateDamageReportCommand,
  UpdateDamageReportCommand,
  DeleteDamageReportCommand,
  UpdateDamageReportRelatedMaintenanceCommand,
} from "./damageReportCommand";

export default abstract class DamageReportCommandHandler {
  /**
   * @description create damageReport
   * @param {CreateDamageReportCommand} createDamageReport
   * @returns {Promise<number>}
   */
  static async create(createDamageReport: CreateDamageReportCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(damageReport)
      .values({
        status: "eingereicht",
        description: createDamageReport.description,
        reportedBy: createDamageReport.reportedBy,
        imageCount: createDamageReport.imageCount,
        equipmentId: createDamageReport.affected == "equipment" ? createDamageReport.affectedId : null,
        vehicleId: createDamageReport.affected == "vehicle" ? createDamageReport.affectedId : null,
        wearableId: createDamageReport.affected == "wearable" ? createDamageReport.affectedId : null,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new DatabaseActionException("CREATE", "damageReport", err);
      });
  }

  /**
   * @description update damageReport
   * @param {UpdateDamageReportCommand} updateDamageReport
   * @returns {Promise<void>}
   */
  static async update(updateDamageReport: UpdateDamageReportCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(damageReport)
      .set({
        status: updateDamageReport.status,
        done: updateDamageReport.done,
      })
      .where("id = :id", { id: updateDamageReport.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("UPDATE", "damageReport", err);
      });
  }

  /**
   * @description update damageReport related maintenance
   * @param {UpdateDamageReportCommand} updateDamageReport
   * @returns {Promise<void>}
   */
  static async updateRelatedMaintenance(
    updateDamageReport: UpdateDamageReportRelatedMaintenanceCommand
  ): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(damageReport)
      .set({
        maintenanceId: updateDamageReport.maintenanceId,
      })
      .where("id = :id", { id: updateDamageReport.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("UPDATE", "damageReport->maintenance", err);
      });
  }

  /**
   * @description delete damageReport
   * @param {DeleteDamageReportCommand} deleteDamageReport
   * @returns {Promise<void>}
   */
  static async delete(deleteDamageReport: DeleteDamageReportCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(damageReport)
      .where("id = :id", { id: deleteDamageReport.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new DatabaseActionException("DELETE", "damageReport", err);
      });
  }
}
